/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2012, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//HttpProxy.java
//Copyright (c) Kawan Softwares S.A.S, 2012
//
//Last Updates: 
// 14 sept. 2010 17:23:52 Kawan Softwares S.A.S
// 20/09/10 20:05 NDP - New header comments
// 04/06/11 12:50 NDP : HttpProxy: Handle NTLM proxy with integrated code of HttpClient 4.1.1
// 08/06/11 12:20 NDP : HttpProxy: Clean Javadoc
// 05/12/11 15:10 NDP : HttpProxy: clean Javadoc

package org.awakefw.commons.api.client;

/**
 * 
 * Allows to define a proxy with or without authentication. All is done through
 * constructor, because instance should be immutable:
 * 
 * <blockquote>
 * 
 * <pre>
 * String url = &quot;https://www.acme.org/AwakeFileManager&quot;;
 * String username = &quot;myUsername&quot;;
 * char[] password = { 'm', 'y', 'P', 'a', 's', 's', 'w', 'o', 'r', 'd' };
 * 
 * HttpProxy httpProxy = null;
 * &nbsp;
 * // Constructor to use if http proxy is without authentication:
 * httpProxy = new HttpProxy(&quot;myProxyhost&quot;, 8080);
 * &nbsp;
 * // Constructor to use if http proxy requires authentication:
 * httpProxy = new HttpProxy(&quot;myProxyhost&quot;, 8080, &quot;username&quot;, &quot;password&quot;);
 * &nbsp;
 * // Constructor to use if http proxy is a NTLM proxy:
 * httpProxy = new HttpProxy(&quot;myProxyhost&quot;, 8080, &quot;username&quot;, &quot;password&quot;,
 * 	&quot;myWorkstation&quot;, &quot;myDomain&quot;);
 * &nbsp;
 * // Open an Awake File Session through a proxy:
 * AwakeFileSession awakeFileSession = new AwakeFileSession(url, username,
 * 	password, httpProxy);
 * &nbsp;
 * // Etc.
 * </pre>
 * 
 * </blockquote>
 * 
 * @author Nicolas de Pomereu
 * @since 1.0
 */

public final class HttpProxy {

    /** The http Proxy Address to use */
    private String address = null;

    /** The http Proxy Port to use */
    private int port = 0;

    /** The proxy credential username */
    private String username = null;

    /** The proxy credential password */
    private String password = null;

    /** NTLM Workstation */
    private String workstation = null;

    /** NTLM Domain */
    private String domain = null;

    /**
     * Constructor to use to define a proxy that does not requires
     * authentication.
     * 
     * @param address
     *            the proxy address
     * @param port
     *            the proxy port
     * 
     * @throws IllegalArgumentException
     *             if address is null
     */
    public HttpProxy(String address, int port) {
	if (address == null) {
	    throw new IllegalArgumentException("Proxy address can not be null!");
	}

	this.address = address;
	this.port = port;
    }

    /**
     * Constructor to use to define a proxy that requires authentication
     * 
     * @param address
     *            the proxy address
     * @param port
     *            the proxy port
     * @param username
     *            the username required for authentication
     * @param password
     *            the password for authentication (maybe null if no password is
     *            required)
     * 
     * 
     * @throws IllegalArgumentException
     *             if address or username is null
     */
    public HttpProxy(String address, int port, String username, String password) {
	this(address, port);

	if (username == null) {
	    throw new IllegalArgumentException(
		    "Proxy username can not be null!");
	}

	this.username = username;
	this.password = password;
    }

    /**
     * Constructor to use to define a Microsoft NTLM proxy that requires
     * authentication.
     * 
     * @param address
     *            the NTLM proxy address
     * @param port
     *            the NTLM proxy port
     * @param username
     *            the username required for authentication
     * @param password
     *            the password for authentication (maybe null if no password is
     *            required)
     * @param workstation
     *            the NTLM workstation parameter
     * @param domain
     *            the NTLM domain parameter
     * 
     * @throws IllegalArgumentException
     *             if address or username is null
     */
    public HttpProxy(String address, int port, String username,
	    String password, String workstation, String domain) {
	this(address, port, username, password);

	if (workstation == null) {
	    throw new IllegalArgumentException(
		    "NTLM Proxy workstation can not be null!");
	}

	if (domain == null) {
	    throw new IllegalArgumentException(
		    "NTLM Proxy domain can not be null!");
	}

	this.workstation = workstation;
	this.domain = domain;
    }

    /**
     * Returns the address of the proxy.
     * 
     * @return the address of the proxy
     */
    public String getAddress() {
	return this.address;
    }

    /**
     * Returns the port of the proxy.
     * 
     * @return the port of the proxy
     */
    public int getPort() {
	return this.port;
    }

    /**
     * Returns the username required for authentication. (<code>null</code> if no
     * authentication is required).
     * 
     * @return the username required for authentication
     */
    public String getUsername() {
	return this.username;
    }

    /**
     * Returns the password. (<code>null</code> if no authentication is required or if no
     * password is required for authentication).
     * 
     * @return the password
     */
    public String getPassword() {
	return this.password;
    }

    /**
     * Returns the NTLM workstation parameter.
     * 
     * @return the NTLM workstation parameter
     */
    public String getWorkstation() {
	return this.workstation;
    }

    /**
     * Returns the NTLM domain parameter.
     * 
     * @return the NTLM domain parameter
     */
    public String getDomain() {
	return this.domain;
    }

    /**
     * Returns a clean representation of the <code>HttpProxy</code> instance.
     * @return a clean representation of the <code>HttpProxy</code> instance
     */
    @Override
    public String toString() {
	return "HttpProxy [address=" + this.address + ", port=" + this.port
		+ ", username=" + this.username + ", password=" + this.password
		+ ", workstation=" + this.workstation + ", domain="
		+ this.domain + "]";
    }

    
    
}
